@@ -5,6 +5,7 @@ import builtins
 import getpass
 import signal
 import sys
+import threading
 import typing as t
 from contextlib import contextmanager
 from functools import partial
@@ -12,10 +13,11 @@ from functools import partial
 import comm
 from IPython.core import release
 from IPython.utils.tokenutil import line_at_cursor, token_at_cursor
-from traitlets import Any, Bool, Instance, List, Type, observe, observe_compat
+from traitlets import Any, Bool, HasTraits, Instance, List, Type, observe, observe_compat
 from zmq.eventloop.zmqstream import ZMQStream
 
 from .comm.comm import BaseComm
+from .comm.manager import CommManager
 from .compiler import XCachingCompiler
 from .debugger import Debugger, _is_debugpy_available
 from .eventloops import _use_appnope
@@ -40,15 +42,33 @@ except ImportError:
 _EXPERIMENTAL_KEY_NAME = "_jupyter_types_experimental"
 
 
-def create_comm(*args, **kwargs):
+def _create_comm(*args, **kwargs):
     """Create a new Comm."""
     return BaseComm(*args, **kwargs)
 
 
-comm.create_comm = create_comm
+# there can only be one comm manager in a ipykernel process
+_comm_lock = threading.Lock()
+_comm_manager: t.Optional[CommManager] = None
+
+
+def _get_comm_manager(*args, **kwargs):
+    """Create a new CommManager."""
+    global _comm_manager
+    if _comm_manager is None:
+        with _comm_lock:
+            if _comm_manager is None:
+                _comm_manager = CommManager(*args, **kwargs)
+    return _comm_manager
+
+
+comm.create_comm = _create_comm
+comm.get_comm_manager = _get_comm_manager
 
 
 class IPythonKernel(KernelBase):
+    """The IPython Kernel class."""
+
     shell = Instance("IPython.core.interactiveshell.InteractiveShellABC", allow_none=True)
     shell_class = Type(ZMQInteractiveShell)
 
@@ -82,6 +102,7 @@ class IPythonKernel(KernelBase):
     _sys_eval_input = Any()
 
     def __init__(self, **kwargs):
+        """Initialize the kernel."""
         super().__init__(**kwargs)
 
         # Initialize the Debugger
@@ -112,6 +133,7 @@ class IPythonKernel(KernelBase):
 
         self.comm_manager = comm.get_comm_manager()
 
+        assert isinstance(self.comm_manager, HasTraits)
         self.shell.configurables.append(self.comm_manager)
         comm_msg_types = ["comm_open", "comm_msg", "comm_close"]
         for msg_type in comm_msg_types:
@@ -181,10 +203,12 @@ class IPythonKernel(KernelBase):
         return self.shell.banner
 
     async def poll_stopped_queue(self):
+        """Poll the stopped queue."""
         while True:
             await self.debugger.handle_stopped_event()
 
     def start(self):
+        """Start the kernel."""
         self.shell.exit_now = False
         if self.debugpy_stream is None:
             self.log.warning("debugpy_stream undefined, debugging will not be enabled")
@@ -316,6 +340,7 @@ class IPythonKernel(KernelBase):
         *,
         cell_id=None,
     ):
+        """Handle code execution."""
         shell = self.shell  # we'll need this a lot here
 
         self._forward_input(allow_stdin)
@@ -450,6 +475,7 @@ class IPythonKernel(KernelBase):
         return reply_content
 
     def do_complete(self, code, cursor_pos):
+        """Handle code completion."""
         if _use_experimental_60_completion and self.use_experimental_completions:
             return self._experimental_do_complete(code, cursor_pos)
 
@@ -470,6 +496,7 @@ class IPythonKernel(KernelBase):
         }
 
     async def do_debug_request(self, msg):
+        """Handle a debug request."""
         if _is_debugpy_available:
             return await self.debugger.process_request(msg)
 
@@ -513,6 +540,7 @@ class IPythonKernel(KernelBase):
         }
 
     def do_inspect(self, code, cursor_pos, detail_level=0, omit_sections=()):
+        """Handle code inspection."""
         name = token_at_cursor(code, cursor_pos)
 
         reply_content: t.Dict[str, t.Any] = {"status": "ok"}
@@ -550,6 +578,7 @@ class IPythonKernel(KernelBase):
         pattern=None,
         unique=False,
     ):
+        """Handle code history."""
         if hist_access_type == "tail":
             hist = self.shell.history_manager.get_tail(
                 n, raw=raw, output=output, include_latest=True
@@ -573,10 +602,12 @@ class IPythonKernel(KernelBase):
         }
 
     def do_shutdown(self, restart):
+        """Handle kernel shutdown."""
         self.shell.exit_now = True
         return dict(status="ok", restart=restart)
 
     def do_is_complete(self, code):
+        """Handle an is_complete request."""
         transformer_manager = getattr(self.shell, "input_transformer_manager", None)
         if transformer_manager is None:
             # input_splitter attribute is deprecated
@@ -588,6 +619,7 @@ class IPythonKernel(KernelBase):
         return r
 
     def do_apply(self, content, bufs, msg_id, reply_metadata):
+        """Handle an apply request."""
         try:
             from ipyparallel.serialize import serialize_object, unpack_apply_message
         except ImportError:
@@ -612,7 +644,7 @@ class IPythonKernel(KernelBase):
             working.update(ns)
             code = f"{resultname} = {fname}(*{argname},**{kwargname})"
             try:
-                exec(code, shell.user_global_ns, shell.user_ns)
+                exec(code, shell.user_global_ns, shell.user_ns)  # noqa
                 result = working.get(resultname)
             finally:
                 for key in ns:
@@ -652,6 +684,7 @@ class IPythonKernel(KernelBase):
         return reply_content, result_buf
 
     def do_clear(self):
+        """Clear the kernel."""
         self.shell.reset(False)
         return dict(status="ok")
 
@@ -660,7 +693,10 @@ class IPythonKernel(KernelBase):
 
 
 class Kernel(IPythonKernel):
+    """DEPRECATED.  An alias for the IPython kernel class."""
+
     def __init__(self, *args, **kwargs):  # pragma: no cover
+        """DEPRECATED."""
         import warnings
 
         warnings.warn(
